@@ -1,9 +1,9 @@
 #ifndef GaMaLib___language__header_file_h
 #define GaMaLib___language__header_file_h
 
-namespace GaMaLib {      /* slovnikar 1.05 */
+namespace GaMaLib {      /* slovnikar 1.07 */
 
-enum gama_language { en, cz, fi, du, hu, ca };
+enum gama_language { en, ca, cz, du, fi, fr, hu, ru };
 void set_gama_language(gama_language);
 
 extern const char* T_GKF_bad_angle;
@@ -224,6 +224,7 @@ extern const char* T_LN_rm_missing_xyz;
 extern const char* T_LN_rm_missing_z;
 extern const char* T_LN_rm_removed_points;
 extern const char* T_LN_rm_singular_xy;
+extern const char* T_LN_rm_singular_z;
 extern const char* T_LN_undefined_confidence_level;
 extern const char* T_LN_undefined_type_of_actual_sigma;
 extern const char* T_POBS_bad_data;
