@@ -20,7 +20,7 @@
 */
 
 /*
- *  $Id: network.h,v 1.14 2005/06/05 15:36:20 cepek Exp $
+ *  $Id: network.h,v 1.16 2005/06/23 19:57:58 cepek Exp $
  */
 
 // LocalNetwork - Network Informations class (Informace o siti)
@@ -61,7 +61,8 @@ namespace GaMaLib
       
       enum rm_points {
         rm_missing_xyz, rm_missing_xy,   rm_missing_z,
-        rm_singular_xy, rm_huge_cov_xyz, rm_huge_cov_xy, rm_huge_cov_z
+        rm_singular_xy, rm_singular_z,
+        rm_huge_cov_xyz, rm_huge_cov_xy, rm_huge_cov_z
       };
       
       GaMaLib::PointIDList  removed_points;
@@ -183,6 +184,7 @@ namespace GaMaLib
       
       Double qxx(Index i, Index j) { return q_xx(i,j); } 
       Double qbb(Index i, Index j) { return q_bb(i,j); } 
+      Double qbx(Index i, Index j) { return q_bx(i,j); } 
 
       virtual Double cond() = 0;
       virtual bool lindep(Index i) = 0;
