@@ -16,17 +16,17 @@
 
     You should have received a copy of the GNU General Public License
     along with this library; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */
 
 /*
- *  $Id: readsabw.h,v 1.2 2004/06/20 20:54:41 cepek Exp $
+ *  $Id: readsabw.h,v 1.4 2005/05/07 18:06:19 cepek Exp $
  */
 
 #ifndef GaMaLib_read_sparse_Ab_h
 #define GaMaLib_read_sparse_Ab_h
 
-#include <gmatvec/gmatvec.h>
+#include <matvec/matvec.h>
 #include <iostream>
 
 namespace GaMaLib {
