@@ -1,6 +1,6 @@
 /*  
     GNU Gama -- adjustment of geodetic networks
-    Copyright (C) 2003  Ales Cepek <cepek@fsv.cvut.cz>
+    Copyright (C) 2003  Ales Cepek <cepek@gnu.org>
 
     This file is part of the GNU Gama C++ library.
     
@@ -20,20 +20,21 @@
 */
 
 /*
- *  $Id: g3_model.cpp,v 1.33 2005/05/07 18:06:20 cepek Exp $
+ *  $Id: g3_model.cpp,v 1.45 2005/09/28 14:35:59 cepek Exp $
  */
 
 #include <gnu_gama/g3/g3_model.h>
 #include <gnu_gama/g3/g3_cluster.h>
 #include <gnu_gama/outstream.h>
 #include <gnu_gama/adj/adj.h>
+#include <iomanip>
 
 
 using namespace std;
 using namespace GNU_gama::g3;
 
 
-Model::Model() 
+Model::Model()
 { 
   using namespace GNU_gama;
 
@@ -45,6 +46,13 @@ Model::Model()
   points->set_common_data(this); 
   set(&ellipsoid, ellipsoid_wgs84);
 
+  set_angular_units_gons();
+
+  apriori_sd       = 1.00;
+  confidence_level = 0.95;
+
+  actual_sd = aposteriori;
+
   reset();
 }
 
@@ -250,7 +258,6 @@ void Model::update_linearization()
 {
   if (!check_observations()) update_observations();
 
-  cerr << "- - - - - - - - - - - - - - - - - - - - - - - .\n";
   adj_input_data = new AdjInputData;
 
   A = new SparseMatrix<>(dm_floats, dm_rows, dm_cols);
@@ -329,19 +336,53 @@ void Model::update_linearization()
 
 void Model::update_adjustment()
 {
-  cerr << "\n************ void Model::update_adjustment()\n\n";
   if (!check_linearization()) update_linearization();
 
-  cerr << "x() = " << adj->x();
+  // parameters 'height' are linked to corresponding parameters 'U'
+
+  for (Model::PointBase::iterator 
+         i=points->begin(), e=points->end(); i!=e; ++i)
+    {
+      Parameter& U      = (*i)->U;
+      Parameter& height = (*i)->height;
+
+      if      (U.fixed() ) height.set_fixed();
+      else if (U.constr()) height.set_constr();
+      else if (U.free()  ) height.set_free();
+      else                 height.set_unused();
+
+      if (height.free())
+        {
+          int k = U.index();
+          height.set_index(k);
+          height.add_correction(adj->x()(k));
+        }
+    }
+
   for (ParameterList::iterator 
          i=par_list->begin(), e=par_list->end(); i!=e; ++i)
     {
       Parameter* p = *i;
-      if (int k = p->index()) p->set_correction(adj->x()(k));
+      if (int k = p->index()) p->add_correction(adj->x()(k));
     }
-  
 
-  cerr << "\n------------ void Model::update_adjustment()\n\n";
+  // ..........................................................
+
+  redundancy = dm_rows - dm_cols + adj->defect();
+
+  aposteriori_sd = 0;
+  if (redundancy) 
+    {
+      const Vec<>& r = adj->r();
+      aposteriori_sd = sqrt(trans(r)*r/redundancy)*1e3;
+    }
+
+  if (actual_sd == apriori)
+    std_deviation = apriori_sd;
+  else
+    std_deviation = aposteriori_sd;
+
+  
   return next_state_(adjust_);
 }
 
@@ -360,9 +401,57 @@ void Model::write_xml_adjustment_results(std::ostream& out)
 {
   if (!check_adjustment()) update_adjustment();
 
-  out << 
-    "\n<!-- adjustment results  :"
-    " dn / de / du  are in millimeters -->\n\n";
+  const Vec<>& r = adj->r();
+
+  out << "\n<adjustmen-statistics>\n\n";
+
+  Adj::algorithm alg = adj->get_algorithm();
+  out << "<algorithm> ";
+  if      (alg == Adj::gso)      cout << "gso";
+  else if (alg == Adj::svd)      cout << "svd";
+  else if (alg == Adj::cholesky) cout << "cholesky";
+  else                           cout << "unknown";
+  out << " </algorithm>\n\n";
+
+  {
+    gama_ellipsoid id = gama_ellipsoid(ellipsoid.id);
+    
+    out.setf(ios_base::fixed, ios_base::floatfield);
+    out << "<ellipsoid> "
+        << "<caption> " << gama_ellipsoid_caption[id] << " </caption>\n";
+    out << "            <id>      " << gama_ellipsoid_id[id] 
+        << "         </id>\n";
+    out.precision(5);
+    out << "            <a>       " << ellipsoid.a() << " </a>\n"; 
+    out.precision(5);
+    out << "            <b>       " << ellipsoid.b() << " </b>\n"; 
+    out << "            </ellipsoid>\n\n"; 
+  }
+
+  out << "<parameters>" << setw(5) << dm_cols       << " </parameters>\n";
+  out << "<equations> " << setw(5) << dm_rows       << " </equations>\n";
+  out << "<defect>    " << setw(5) << adj->defect() << " </defect>\n";
+
+  out << "<redundancy>" << setw(5) << redundancy    << " </redundancy\n\n";
+
+  out.setf(ios_base::scientific, ios_base::floatfield);
+  out.precision(5);
+  double rtr = trans(r)*r;
+  out << "<sum-of-squares>    " << rtr << " </sum-of-squares>\n";
+
+  double sigma_apriori = apriori_sd*apriori_sd;
+  out << "<sigma-apriori>     " << sigma_apriori << " </sigma-apriori>\n";
+
+  double sigma_aposteriori = aposteriori_sd*aposteriori_sd;
+  out << "<sigma-aposteriori> "<<sigma_aposteriori<<" </sigma-aposteriori>\n";
+  
+  out << "\n</adjustmen-statistics>\n\n";
+
+  // -----------------------------------------------------------------------
+
+  out << "\n"
+    "<!-- adjustment results    : dn / de / du  are in millimeters -->\n"
+    "<!-- deflection of vertical: db / dl       are in arc seconds -->\n\n";
   
   out << "<adjustment-results>\n";
 
@@ -413,3 +502,15 @@ GNU_gama::E_3 Model::vertical(const Point* p) const
                        std::sin(B)            );
 }
 
+GNU_gama::E_3 Model::instrument(const Point* p, double dh) const
+{
+  using GNU_gama::E_3;
+
+  E_3 s (p->X(), p->Y(), p->Z());
+  E_3 v = vertical(p);
+  v *= dh;
+  s += v;
+
+  return s;
+}
+
