@@ -1,6 +1,6 @@
 /*  
     GNU Gama -- adjustment of geodetic networks
-    Copyright (C) 2003  Ales Cepek <cepek@fsv.cvut.cz>
+    Copyright (C) 2003  Ales Cepek <cepek@gnu.org>
 
     This file is part of the GNU Gama C++ library.
     
@@ -20,7 +20,7 @@
 */
 
 /*
- *  $Id: g3_model.h,v 1.28 2005/05/07 18:06:20 cepek Exp $
+ *  $Id: g3_model.h,v 1.37 2005/09/28 14:35:59 cepek Exp $
  */
 
 #ifndef GNU_gama__g3_model_h_gnugamag3modelh___gnu_gama_g3model
@@ -51,24 +51,29 @@ namespace GNU_gama {  namespace g3 {
     public Revision     <ZenithAngle>,
     public Linearization<ZenithAngle>,
     public Revision     <Azimuth>,
-    public Linearization<Azimuth>
+    public Linearization<Azimuth>,
+    public Revision     <HeightDiff>,
+    public Linearization<HeightDiff>,
+    public Revision     <Height>,
+    public Linearization<Height>,
+    public Revision     <Angle>,
+    public Linearization<Angle>
   {
   public:
     
+    Model();
+    virtual ~Model();    
+    
     typedef GNU_gama::ObservationData<g3::Observation>  ObservationData;
     typedef GNU_gama::List<Observation*>                ObservationList; 
     typedef GNU_gama::PointBase<g3::Point>              PointBase;
     typedef GNU_gama::List<Parameter*>                  ParameterList;
     typedef GNU_gama::Adj                               Adj;
 
+
     PointBase           *points;    
     GNU_gama::Ellipsoid  ellipsoid;
     
-    
-    Model();
-    virtual ~Model();
-    
-    
     Point* get_point(const Point::Name&);
     void   write_xml(std::ostream& out) const;
     
@@ -98,13 +103,35 @@ namespace GNU_gama {  namespace g3 {
     void linearization_visit(ZenithAngle*);
     bool revision_visit     (Azimuth*    );
     void linearization_visit(Azimuth*    );
+    bool revision_visit     (HeightDiff* );
+    void linearization_visit(HeightDiff* );
+    bool revision_visit     (Height*     );
+    void linearization_visit(Height*     );
+    bool revision_visit     (Angle*      );
+    void linearization_visit(Angle*      );
+
+    void set_algorithm(Adj::algorithm a) { adj->set_algorithm(a); }
+
+    void   set_apriori_sd(double s) { apriori_sd = s;          }
+    double get_apriori_sd() const   { return apriori_sd;       } 
+    void   set_conf_level(double c) { confidence_level = c;    }
+    double get_conf_level() const   { return confidence_level; } 
+
+    double standard_deviation() const { return std_deviation; }
+    double q_xx(Index i, Index j) { return adj->q_xx(i,j); }
 
     void write_xml_adjustment_input_data(std::ostream&);
     void write_xml_adjustment_results   (std::ostream&);
 
-    GNU_gama::E_3 vector  (const Point* from, const Point* to) const;
-    GNU_gama::E_3 normal  (const Point* p) const;
-    GNU_gama::E_3 vertical(const Point* p) const;
+    bool angular_units_degrees() const { return !gons_; }
+    bool angular_units_gons   () const { return  gons_; }
+    void set_angular_units_degrees()   { gons_ = false; }
+    void set_angular_units_gons()      { gons_ = true; }
+    
+    GNU_gama::E_3 vector    (const Point* from, const Point* to) const;
+    GNU_gama::E_3 normal    (const Point* p) const;
+    GNU_gama::E_3 vertical  (const Point* p) const;
+    GNU_gama::E_3 instrument(const Point* p, double dh) const;
 
   private:   /*-----------------------------------------------------------*/
       
@@ -136,6 +163,18 @@ namespace GNU_gama {  namespace g3 {
 
     // adjustment
     Adj*              adj;
+
+    int    redundancy;
+    enum { apriori, aposteriori } actual_sd;
+    double aposteriori_sd;
+    double std_deviation;
+    
+
+    // constants
+    double apriori_sd;
+    double confidence_level;
+
+    bool   gons_;
   };
   
 }}
