@@ -1,6 +1,6 @@
 /*  
     GNU Gama -- adjustment of geodetic networks
-    Copyright (C) 2003  Ales Cepek <cepek@fsv.cvut.cz>
+    Copyright (C) 2003  Ales Cepek <cepek@gnu.org>
 
     This file is part of the GNU Gama C++ library.
     
@@ -20,7 +20,7 @@
 */
 
 /*
- *  $Id: g3_model_vector.cpp,v 1.8 2005/05/07 18:06:20 cepek Exp $
+ *  $Id: g3_model_vector.cpp,v 1.10 2005/09/23 17:17:29 cepek Exp $
  */
 
 #include <gnu_gama/g3/g3_model.h>
@@ -105,10 +105,12 @@ void Model::linearization_visit(Vector* v)
      double dx = to->X_dh(v->to_dh) - from->X_dh(v->from_dh);
      double dy = to->Y_dh(v->to_dh) - from->Y_dh(v->from_dh);
      double dz = to->Z_dh(v->to_dh) - from->Z_dh(v->from_dh);
+
+     const double s = Linear().scale();
  
-     rhs(++rhs_ind) = v->dx() - dx;
-     rhs(++rhs_ind) = v->dy() - dy;
-     rhs(++rhs_ind) = v->dz() - dz;
+     rhs(++rhs_ind) = (v->dx() - dx)*s;
+     rhs(++rhs_ind) = (v->dy() - dy)*s;
+     rhs(++rhs_ind) = (v->dz() - dz)*s;
    }
 }
 
