@@ -1,6 +1,6 @@
 /*  
     GNU Gama -- adjustment of geodetic networks
-    Copyright (C) 2003  Ales Cepek <cepek@fsv.cvut.cz>
+    Copyright (C) 2003  Ales Cepek <cepek@gnu.org>
 
     This file is part of the GNU Gama C++ library.
     
@@ -20,7 +20,7 @@
 */
 
 /*
- *  $Id: g3_model_xyz.cpp,v 1.5 2005/05/07 18:06:20 cepek Exp $
+ *  $Id: g3_model_xyz.cpp,v 1.7 2005/09/23 17:17:29 cepek Exp $
  */
 
 #include <gnu_gama/g3/g3_model.h>
@@ -85,10 +85,12 @@ void Model::linearization_visit(XYZ* xyz)
      double x = point->X();
      double y = point->Y();
      double z = point->Z();
-   
-     rhs(++rhs_ind) = xyz->x() - x;
-     rhs(++rhs_ind) = xyz->y() - y;
-     rhs(++rhs_ind) = xyz->z() - z;
+
+     const double s = Linear().scale();
+
+     rhs(++rhs_ind) = (xyz->x() - x)*s;
+     rhs(++rhs_ind) = (xyz->y() - y)*s;
+     rhs(++rhs_ind) = (xyz->z() - z)*s;
    }
 }
 
