@@ -1,6 +1,6 @@
 /*  
     GNU Gama -- adjustment of geodetic networks
-    Copyright (C) 2003  Ales Cepek <cepek@fsv.cvut.cz>
+    Copyright (C) 2003  Ales Cepek <cepek@gnu.org>
 
     This file is part of the GNU Gama C++ library.
     
@@ -20,7 +20,7 @@
 */
 
 /*
- *  $Id: g3_observation.h,v 1.14 2005/05/07 18:06:20 cepek Exp $
+ *  $Id: g3_observation.h,v 1.18 2005/09/28 14:35:59 cepek Exp $
  */
 
 
@@ -246,6 +246,104 @@ namespace GNU_gama {  namespace g3 {
     double x_, y_, z_;
   };
 
+
+  class HeightDiff : public Observation, public FromTo, public Value {
+  public:  
+
+    HeightDiff() {}
+    HeightDiff(double d) : Value(d) {}
+
+    int dimension() const { return 1; }
+
+    bool revision_accept(ObservationVisitor* visitor)
+    {
+      if (Revision<HeightDiff>* 
+          rv = dynamic_cast<Revision<HeightDiff>*>(visitor))
+        {          
+          return rv->revision_visit(this);
+        }
+      else
+        return  set_active(false);
+    }
+  
+    void linearization_accept(ObservationVisitor* visitor)
+    {
+      if (Linearization<HeightDiff>* 
+          lv = dynamic_cast<Linearization<HeightDiff>*>(visitor))
+        {
+          lv->linearization_visit(this);
+        }
+    }
+  };
+
+
+  class Height : public Observation, public Value {
+  public:  
+
+    Point::Name id;
+
+    Height() {}
+    Height(double d) : Value(d) {}
+
+    int dimension() const { return 1; }
+
+    bool revision_accept(ObservationVisitor* visitor)
+    {
+      if (Revision<Height>* 
+          rv = dynamic_cast<Revision<Height>*>(visitor))
+        {          
+          return rv->revision_visit(this);
+        }
+      else
+        return  set_active(false);
+    }
+  
+    void linearization_accept(ObservationVisitor* visitor)
+    {
+      if (Linearization<Height>* 
+          lv = dynamic_cast<Linearization<Height>*>(visitor))
+        {
+          lv->linearization_visit(this);
+        }
+    }
+  };
+
+
+  class Angle : public Observation, public Value {
+  public:
+
+    Point::Name from;
+    Point::Name left;
+    Point::Name right;
+
+    double from_dh, left_dh, right_dh;
+
+    Angle() {}
+    Angle(double d) : Value(d) {}
+
+    int dimension() const { return 1; }
+
+    bool revision_accept(ObservationVisitor* visitor)
+    {
+      if (Revision<Angle>* 
+          rv = dynamic_cast<Revision<Angle>*>(visitor))
+        {          
+          return rv->revision_visit(this);
+        }
+      else
+        return  set_active(false);
+    }
+  
+    void linearization_accept(ObservationVisitor* visitor)
+    {
+      if (Linearization<Angle>* 
+          lv = dynamic_cast<Linearization<Angle>*>(visitor))
+        {
+          lv->linearization_visit(this);
+        }
+    }
+  };
+
 }}
 
 #endif
