@@ -1,6 +1,6 @@
 /*
     Geodesy and Mapping C++ Library (GNU GaMa / GaMaLib)
-    Copyright (C) 2002  Ales Cepek <cepek@fsv.cvut.cz>
+    Copyright (C) 2002, 2005  Ales Cepek <cepek@gnu.org>
 
     This file is part of the GNU GaMa / GaMaLib C++ Library.
     
@@ -20,7 +20,7 @@
 */
 
 /*
- *  $Id: dataparser.h,v 1.23 2005/05/07 18:06:20 cepek Exp $
+ *  $Id: dataparser.h,v 1.31 2005/09/28 14:35:59 cepek Exp $
  */
 
 #ifndef GNU_Gama_GaMa_XML_DataParser__data_parser__dataparser___h_
@@ -76,6 +76,19 @@ namespace GNU_gama {
 
           // ..................................................
 
+          s_g3_const,
+          s_g3_const_apriori_sd,
+          s_g3_const_conf_level,
+          s_g3_const_ellipsoid,
+          s_g3_const_ellipsoid2,
+          s_g3_const_ellipsoid_id,
+          s_g3_const_ellipsoid_a,
+          s_g3_const_ellipsoid_b,
+          s_g3_const_ellipsoid_inv_f,
+          s_g3_const_ang,
+
+          // ..................................................
+
           s_g3_param,
           s_g3_param_n,
           s_g3_param_e,
@@ -97,10 +110,14 @@ namespace GNU_gama {
           s_g3_point_after_y,
           s_g3_point_z,
           s_g3_point_height,
+          s_g3_point_geoid,
           s_g3_point_param,
           s_g3_point_param_n,
           s_g3_point_param_e,
           s_g3_point_param_u,
+          s_g3_point_db,
+          s_g3_point_after_db,
+          s_g3_point_dl,
 
           // ..................................................
 
@@ -124,6 +141,18 @@ namespace GNU_gama {
           s_g3_obs_dist_opt_from_dh,
           s_g3_obs_dist_opt_to_dh,
 
+          s_g3_obs_hdiff,
+          s_g3_obs_hdiff_from,
+          s_g3_obs_hdiff_after_from,
+          s_g3_obs_hdiff_to,
+          s_g3_obs_hdiff_after_to,
+          s_g3_obs_hdiff_val,
+          s_g3_obs_hdiff_opt,
+          s_g3_obs_hdiff_opt_stdev,
+          s_g3_obs_hdiff_opt_variance,
+          s_g3_obs_hdiff_opt_from_dh,
+          s_g3_obs_hdiff_opt_to_dh,
+
           s_g3_obs_zenith,
           s_g3_obs_zenith_from,
           s_g3_obs_zenith_after_from,
@@ -172,6 +201,29 @@ namespace GNU_gama {
           s_g3_obs_xyz_z,
           s_g3_obs_xyz_after_z,
 
+          s_g3_obs_height,
+          s_g3_obs_height_id,
+          s_g3_obs_height_after_id,
+          s_g3_obs_height_val,
+          s_g3_obs_height_opt,
+          s_g3_obs_height_opt_stdev,
+          s_g3_obs_height_opt_variance,
+
+          s_g3_obs_angle,
+          s_g3_obs_angle_from,
+          s_g3_obs_angle_after_from,
+          s_g3_obs_angle_left,
+          s_g3_obs_angle_after_left,
+          s_g3_obs_angle_right,
+          s_g3_obs_angle_after_right,
+          s_g3_obs_angle_val,
+          s_g3_obs_angle_opt,
+          s_g3_obs_angle_opt_stdev,
+          s_g3_obs_angle_opt_variance,
+          s_g3_obs_angle_opt_from_dh,
+          s_g3_obs_angle_opt_left_dh,
+          s_g3_obs_angle_opt_right_dh,
+          
           // ..................................................
 
           s_text,
@@ -224,7 +276,12 @@ namespace GNU_gama {
       
       enum data_tag 
         {
+          t_a,
           t_adj_input_data,
+          t_ang_degrees,
+          t_ang_gons,
+          t_angle,
+          t_apriori_sd,
           t_array,
           t_azimuth,
           t_b,
@@ -233,14 +290,19 @@ namespace GNU_gama {
           t_block_diagonal,
           t_blocks,
           t_cols,
+          t_conf_level,
+          t_constants,
           t_constr,
           t_covmat,
+          t_db,
+          t_dl,
           t_dim,
           t_dist,
           t_dx,
           t_dy,
           t_dz,
           t_e,
+          t_ellipsoid,
           t_flt,
           t_fixed,
           t_free,
@@ -248,15 +310,22 @@ namespace GNU_gama {
           t_from_dh,
           t_g3_model,
           t_gama_data,
+          t_geoid,
           t_h,
+          t_hdiff,
           t_height,
           t_id,
           t_int,
+          t_inv_f,
           t_l,
+          t_left,
+          t_left_dh,
           t_n,
           t_nonz,
           t_obs,
           t_point,
+          t_right,
+          t_right_dh,
           t_rows,
           t_row,
           t_sparse_mat,
@@ -291,60 +360,73 @@ namespace GNU_gama {
       int next [s_stop+1][t_unknown+1];
       int after[s_stop+1]; 
 
-      int gama_data             (const char *name, const char **atts);
-      int g3_model              (const char *name, const char **atts);
-      int g3_model              (const char *name);
+      int gama_data               (const char *name, const char **atts);
+      int g3_model                (const char *name, const char **atts);
+      int g3_model                (const char *name);
 
-      int g3_param_unused       (const char *name, const char **atts);
-      int g3_param_fixed        (const char *name, const char **atts);
-      int g3_param_free         (const char *name, const char **atts);
-      int g3_param_constr       (const char *name, const char **atts);
+      int g3_const_apriori_sd     (const char *name);
+      int g3_const_conf_level     (const char *name);
+      int g3_const_ellipsoid_id   (const char *name);
+      int g3_const_ellipsoid_b    (const char *name);
+      int g3_const_ellipsoid_inv_f(const char *name);
+      int g3_const_ang_degrees    (const char *name);
+      int g3_const_ang_gons       (const char *name);
 
-      int g3_param_n            (const char *name);
-      int g3_param_e            (const char *name);
-      int g3_param_u            (const char *name);
-
-      int g3_point              (const char *name);
-      int g3_point_id           (const char *name);
-      int g3_point_b            (const char *name);
-      int g3_point_l            (const char *name);
-      int g3_point_h            (const char *name);
-      int g3_point_z            (const char *name);
-      int g3_point_height       (const char *name);
-      int g3_point_param_n      (const char *name);
-      int g3_point_param_e      (const char *name);
-      int g3_point_param_u      (const char *name);
-
-      int g3_obs                (const char *name, const char **atts);
-      int g3_obs                (const char *name);
-      int g3_obs_cov            (const char *name);
-      int g3_obs_dist           (const char *name);
-      int g3_obs_zenith         (const char *name);
-      int g3_obs_azimuth        (const char *name);
-      int g3_obs_vector         (const char *name);
-      int g3_obs_xyz            (const char *name);
-
-      int text                  (const char *name);
-
-      int adj_input_data        (const char *name, const char **atts);
-      int adj_input_data        (const char *name);
-      int sparse_mat            (const char *name);
-      int sparse_mat_nonz       (const char *name);
-      int sparse_mat_row        (const char *name, const char **atts);
-      int sparse_mat_row        (const char *name);
-      int sparse_mat_row_n      (const char *name);
-      int sparse_mat_row_f      (const char *name);
-      int block_diagonal        (const char *name);
-      int block_diagonal_nonz   (const char *name);
-      int block_diagonal_block_w(const char *name);
-      int block_diagonal_vec_flt(const char *name);
-      int block_diagonal_block  (const char *name);
-      int vector                (const char *name);
-      int vector_dim            (const char *name);
-      int vector_flt            (const char *name);
-      int array                 (const char *name);
-      int array_dim             (const char *name);
-      int array_int             (const char *name);
+      int g3_param_unused         (const char *name, const char **atts);
+      int g3_param_fixed          (const char *name, const char **atts);
+      int g3_param_free           (const char *name, const char **atts);
+      int g3_param_constr         (const char *name, const char **atts);
+                                  
+      int g3_param_n              (const char *name);
+      int g3_param_e              (const char *name);
+      int g3_param_u              (const char *name);
+                                  
+      int g3_point                (const char *name);
+      int g3_point_id             (const char *name);
+      int g3_point_b              (const char *name);
+      int g3_point_l              (const char *name);
+      int g3_point_h              (const char *name);
+      int g3_point_z              (const char *name);
+      int g3_point_height         (const char *name);
+      int g3_point_geoid          (const char *name);
+      int g3_point_param_n        (const char *name);
+      int g3_point_param_e        (const char *name);
+      int g3_point_param_u        (const char *name);
+      int g3_point_dl             (const char *name);
+                                  
+      int g3_obs                  (const char *name, const char **atts);
+      int g3_obs                  (const char *name);
+      int g3_obs_cov              (const char *name);
+      int g3_obs_dist             (const char *name);
+      int g3_obs_zenith           (const char *name);
+      int g3_obs_azimuth          (const char *name);
+      int g3_obs_vector           (const char *name);
+      int g3_obs_xyz              (const char *name);
+      int g3_obs_hdiff            (const char *name);
+      int g3_obs_height           (const char *name);
+      int g3_obs_angle            (const char *name);
+                                  
+      int text                    (const char *name);
+                                  
+      int adj_input_data          (const char *name, const char **atts);
+      int adj_input_data          (const char *name);
+      int sparse_mat              (const char *name);
+      int sparse_mat_nonz         (const char *name);
+      int sparse_mat_row          (const char *name, const char **atts);
+      int sparse_mat_row          (const char *name);
+      int sparse_mat_row_n        (const char *name);
+      int sparse_mat_row_f        (const char *name);
+      int block_diagonal          (const char *name);
+      int block_diagonal_nonz     (const char *name);
+      int block_diagonal_block_w  (const char *name);
+      int block_diagonal_vec_flt  (const char *name);
+      int block_diagonal_block    (const char *name);
+      int vector                  (const char *name);
+      int vector_dim              (const char *name);
+      int vector_flt              (const char *name);
+      int array                   (const char *name);
+      int array_dim               (const char *name);
+      int array_int               (const char *name);
       
       int add_text     (const char *name, int len);
       int end_tag      (const char *name);
@@ -357,6 +439,8 @@ namespace GNU_gama {
       int optional_variance(const char *name, int len);
       int optional_from_dh (const char *name, int len);
       int optional_to_dh   (const char *name, int len);
+      int optional_left_dh (const char *name, int len);
+      int optional_right_dh(const char *name, int len);
 
       void init(int state, int tag, 
                 int next_state, int end_state, int after_state,
