@@ -1,8 +1,8 @@
 /*  
     Geodesy and Mapping C++ Library (GNU GaMa / GaMaLib)
-    Copyright (C) 2002  Ales Cepek <cepek@fsv.cvut.cz>
+    Copyright (C) 2002, 2005  Ales Cepek <cepek@gnu.org>
 
-    This file is part of the GNU GaMa / GaMaLib C++ Library.
+    This file is part of the GNU Gama C++ library
     
     This library is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -20,7 +20,7 @@
 */
 
 /*
- *  $Id: dataparser_g3.cpp,v 1.6 2005/05/07 18:06:20 cepek Exp $
+ *  $Id: dataparser_g3.cpp,v 1.15 2005/09/28 14:35:59 cepek Exp $
  */
 
 
@@ -45,6 +45,10 @@ namespace GNU_gama {
       delete model;
     }
 
+    typedef std::list<double> Scale;
+    Scale              scale;
+
+
     typedef g3::Model::ObservationType::CovarianceMatrix Cov;
 
     g3::Model*         model;
@@ -52,6 +56,8 @@ namespace GNU_gama {
     std::list<Cov>     cov_list;
     double             from_dh;
     double             to_dh;
+    double             left_dh;
+    double             right_dh;
   };
 
 }
@@ -76,6 +82,50 @@ void DataParser::init_g3()
        s_g3_model, 0, 0,
        &DataParser::g3_model, 0, &DataParser::g3_model);
 
+  // .....  <g3-model>  <constants>  .................................
+
+  init(s_g3_model, t_constants,
+       s_g3_const, 0, s_g3_model,
+       0, 0, 0);
+
+  init(s_g3_const, t_apriori_sd,
+       s_g3_const_apriori_sd, 0, s_g3_const,
+       0, &DataParser::add_text, &DataParser::g3_const_apriori_sd);
+
+  init(s_g3_const, t_conf_level,
+       s_g3_const_conf_level, 0, s_g3_const,
+       0, &DataParser::add_text, &DataParser::g3_const_conf_level);
+
+  init(s_g3_const, t_ang_degrees,
+       s_g3_const_ang, 0, s_g3_const,
+       0, 0, &DataParser::g3_const_ang_degrees);
+
+  init(s_g3_const, t_ang_gons,
+       s_g3_const_ang, 0, s_g3_const,
+       0, 0, &DataParser::g3_const_ang_gons);
+
+  // .....  <g3-model>  <constants>  <ellipsoid>  ....................
+
+  init(s_g3_const, t_ellipsoid,
+       s_g3_const_ellipsoid, s_g3_const_ellipsoid2, s_g3_const,
+       0, 0, 0);
+
+  init(s_g3_const_ellipsoid, t_id,
+       s_g3_const_ellipsoid_id, 0, s_g3_const_ellipsoid2,
+       0, &DataParser::add_text, &DataParser::g3_const_ellipsoid_id);
+
+  init(s_g3_const_ellipsoid, t_a,
+       0, 0, s_g3_const_ellipsoid_a,
+       0, &DataParser::add_text, 0);
+  
+  init(s_g3_const_ellipsoid_a, t_b,
+       s_g3_const_ellipsoid_b, 0, s_g3_const_ellipsoid2,
+       0, &DataParser::add_text, &DataParser::g3_const_ellipsoid_b);
+
+  init(s_g3_const_ellipsoid_a, t_inv_f,
+       s_g3_const_ellipsoid_inv_f, 0, s_g3_const_ellipsoid2,
+       0, &DataParser::add_text, &DataParser::g3_const_ellipsoid_inv_f);
+
   // .....  <g3-model> <unused | fixed | free | constr>  .............
 
   init(s_g3_model, t_unused,
@@ -145,6 +195,10 @@ void DataParser::init_g3()
        s_g3_point_height, 0, s_g3_point_2,
        0, &DataParser::add_text, &DataParser::g3_point_height);
 
+  init(s_g3_point_2, t_geoid,
+       s_g3_point_geoid, 0, s_g3_point_2,
+       0, &DataParser::add_text, &DataParser::g3_point_geoid);
+
   init(s_g3_point_2, t_unused,
        s_g3_point_param, 0, s_g3_point_2,
        &DataParser::g3_param_unused, 0, 0);
@@ -173,6 +227,14 @@ void DataParser::init_g3()
        s_g3_point_param_u, 0, 0,
        0, 0, &DataParser::g3_point_param_u);
 
+  init(s_g3_point_2, t_db,
+       s_g3_point_db, 0, s_g3_point_after_db,
+       0, &DataParser::add_text, 0);
+
+  init(s_g3_point_after_db, t_dl,
+       s_g3_point_dl, 0, s_g3_point_2,
+       0, &DataParser::add_text, &DataParser::g3_point_dl);
+
   // .....  <g3-model> <obs>  ........................................
   
   init(s_g3_model, t_obs,
@@ -333,7 +395,7 @@ void DataParser::init_g3()
         s_g3_obs_vector_opt_to_dh, 0, 0,
         0, &DataParser::optional_to_dh, 0);
 
-  // .....  <g3-model> <obs> <xyz>  ..................................
+   // .....  <g3-model> <obs> <xyz>  ..................................
 
    init(s_g3_obs, t_xyz,
         s_g3_obs_xyz, s_g3_obs_xyz_after_z, 0,
@@ -354,6 +416,104 @@ void DataParser::init_g3()
    init(s_g3_obs_xyz_after_y, t_z,
         s_g3_obs_xyz_z, 0, s_g3_obs_xyz_after_z,
         0, &DataParser::add_text, 0);
+
+   // .....  <g3-model> <obs> <hdiff>  ................................
+
+   init(s_g3_obs, t_hdiff,
+        s_g3_obs_hdiff, s_g3_obs_hdiff_opt, 0,
+        0, 0, &DataParser::g3_obs_hdiff);
+   
+   init(s_g3_obs_hdiff, t_from,
+        s_g3_obs_hdiff_from, 0, s_g3_obs_hdiff_after_from,
+        0, &DataParser::add_text, 0);
+
+   init(s_g3_obs_hdiff_after_from, t_to,
+        s_g3_obs_hdiff_to, 0, s_g3_obs_hdiff_after_to,
+        0, &DataParser::add_text, 0);
+   
+   init(s_g3_obs_hdiff_after_to, t_val,
+        s_g3_obs_hdiff_val, 0, s_g3_obs_hdiff_opt,
+        0, &DataParser::add_text, 0);
+   
+   init(s_g3_obs_hdiff_opt, t_stdev,
+        s_g3_obs_hdiff_opt_stdev, 0, 0,
+        0, &DataParser::optional_stdev, 0);
+   
+   init(s_g3_obs_hdiff_opt, t_variance,
+        s_g3_obs_hdiff_opt_variance, 0, 0,
+        0, &DataParser::optional_variance, 0);
+ 
+//  init(s_g3_obs_hdiff_opt, t_from_dh,
+//       s_g3_obs_hdiff_opt_from_dh, 0, 0,
+//       0, &DataParser::optional_from_dh, 0);
+// 
+//  init(s_g3_obs_hdiff_opt, t_to_dh,
+//       s_g3_obs_hdiff_opt_to_dh, 0, 0,
+//       0, &DataParser::optional_to_dh, 0);
+
+   // ..... <g3-model> <obs> <height> ................................... 
+
+   init(s_g3_obs, t_height,
+        s_g3_obs_height, s_g3_obs_height_opt, 0,
+        0, 0, &DataParser::g3_obs_height);
+   
+   init(s_g3_obs_height, t_id,
+        s_g3_obs_height_id, 0, s_g3_obs_height_after_id,
+        0, &DataParser::add_text, 0);
+
+   init(s_g3_obs_height_after_id, t_val,
+        s_g3_obs_height_val, 0, s_g3_obs_height_opt,
+        0, &DataParser::add_text, 0);
+   
+   init(s_g3_obs_height_opt, t_stdev,
+        s_g3_obs_height_opt_stdev, 0, 0,
+        0, &DataParser::optional_stdev, 0);
+   
+   init(s_g3_obs_height_opt, t_variance,
+        s_g3_obs_height_opt_variance, 0, 0,
+        0, &DataParser::optional_variance, 0);
+
+   // ..... <g3-model> <obs> <angle> ................................... 
+
+   init(s_g3_obs, t_angle,
+        s_g3_obs_angle, s_g3_obs_angle_opt, 0,
+        0, 0, &DataParser::g3_obs_angle);
+
+   init(s_g3_obs_angle, t_from,
+        s_g3_obs_angle_from, 0, s_g3_obs_angle_after_from,
+        0, &DataParser::add_text, 0);
+
+   init(s_g3_obs_angle_after_from, t_left,
+        s_g3_obs_angle_left, 0, s_g3_obs_angle_after_left,
+        0, &DataParser::add_text, 0);
+
+   init(s_g3_obs_angle_after_left, t_right,
+        s_g3_obs_angle_right, 0, s_g3_obs_angle_after_right,
+        0, &DataParser::add_text, 0);
+
+   init(s_g3_obs_angle_after_right, t_val,
+        s_g3_obs_angle_val, 0, s_g3_obs_angle_opt,
+        0, &DataParser::add_text, 0);
+
+   init(s_g3_obs_angle_opt, t_stdev,
+        s_g3_obs_angle_opt_stdev, 0, 0,
+        0, &DataParser::optional_stdev, 0);
+   
+   init(s_g3_obs_angle_opt, t_variance,
+        s_g3_obs_angle_opt_variance, 0, 0,
+        0, &DataParser::optional_variance, 0);
+   
+   init(s_g3_obs_angle_opt, t_from_dh,
+        s_g3_obs_angle_opt_from_dh, 0, 0,
+        0, &DataParser::optional_from_dh, 0);
+   
+   init(s_g3_obs_angle_opt, t_left_dh,
+        s_g3_obs_angle_opt_left_dh, 0, 0,
+        0, &DataParser::optional_left_dh, 0);
+
+   init(s_g3_obs_angle_opt, t_right_dh,
+        s_g3_obs_angle_opt_right_dh, 0, 0,
+        0, &DataParser::optional_right_dh, 0);
 }
 
 int DataParser::g3_model(const char *name, const char **atts)
@@ -568,12 +728,47 @@ int DataParser::g3_point_height(const char *name)
   return  end_tag(name);
 }
 
+int DataParser::g3_point_geoid(const char *name)
+{
+  stringstream istr(text_buffer);
+  double g;
+
+  if (!(istr >> g))
+    {
+      return error("### bad format of numerical data in <point> geoid ");
+    }
+
+  text_buffer.erase();
+
+  point->set_geoid(g);
+
+  return  end_tag(name);
+}
+
+int DataParser::g3_point_dl(const char *name)
+{
+  stringstream istr(text_buffer);
+  double db, dl;
+
+  if (!(istr >> db >> dl))
+    {
+      return error("### bad format of numerical data in <point> db dl ");
+    }
+  text_buffer.erase();
+
+  point->dB.set_init_value(db*SS_TO_RAD);
+  point->dL.set_init_value(dl*SS_TO_RAD);
+
+  return  end_tag(name);
+}
+
 int DataParser::g3_obs(const char *name, const char **atts)
 {
   no_attributes( name, atts );
   state = next[state][tag(name)];
 
   g3->obs_cluster = new g3::ObsCluster(&g3->model->obsdata);
+  g3->scale.clear();
 
   return 0;
 }
@@ -590,6 +785,11 @@ int DataParser::g3_obs(const char *name)
       obs_dim += (*i)->dimension();
     }
 
+  if (obs_dim != g3->scale.size()) 
+    return error("### INTERNAL ERROR IN "
+                 "int DataParser::g3_obs(const char *name)");
+
+
   int cov_dim  = 0;
   int cov_band = 0; 
   typedef std::list<CovMat<> >::const_iterator Iterator;
@@ -624,6 +824,19 @@ int DataParser::g3_obs(const char *name)
     if(g3->obs_cluster->covariance_matrix(i,i) <= 0)
       return error("### zero or negative variance");
 
+
+  // here we scale covariance matrix so that all its elements have the
+  // same units as their corresponding internal representation of
+  // observables (linear data are stored in meters, angular values in
+  // radians)
+
+  DataParser_g3::Scale::const_iterator s = g3->scale.begin();
+  for (int i=1; i<=obs_dim; i++, s++)
+    {
+      if (*s != 1.0) g3->obs_cluster->scaleCov(i, *s);
+    }
+  g3->scale.clear();
+
   g3->obs_cluster->update();
   g3->model->obsdata.clusters.push_back(g3->obs_cluster);
   g3->cov_list.clear();
@@ -706,6 +919,26 @@ int DataParser::optional_to_dh(const char *s, int len)
   return error("### bad data in <to-dh>");
 }
 
+int DataParser::optional_left_dh(const char *s, int len)
+{
+  using namespace g3;
+  stringstream istr(string(s,len));
+
+  if (pure_data(istr >> g3->left_dh)) return 0;
+
+  return error("### bad data in <left-dh>");
+}
+
+int DataParser::optional_right_dh(const char *s, int len)
+{
+  using namespace g3;
+  stringstream istr(string(s,len));
+
+  if (pure_data(istr >> g3->right_dh)) return 0;
+
+  return error("### bad data in <right-dh>");
+}
+
 int DataParser::g3_obs_dist(const char *name)
 {
   using namespace g3;  
@@ -724,6 +957,7 @@ int DataParser::g3_obs_dist(const char *name)
       distance->from_dh = optional(g3->from_dh);
       distance->to_dh   = optional(g3->to_dh);
       g3->obs_cluster->observation_list.push_back(distance);  
+      g3->scale.push_back(1.0);
 
       return  end_tag(name);
     }
@@ -743,16 +977,22 @@ int DataParser::g3_obs_zenith(const char *name)
       text_buffer.clear();
 
       double val;
-      if (!deg2gon(sval, val))
+      if (deg2gon(sval, val))
+        {
+          g3->scale.push_back(3.0864); // ss --> cc
+        }
+      else
         {
           istringstream istr(sval);
           istr >> val;
+
+          g3->scale.push_back(1.0);
         }
 
       ZenithAngle* zenith = new ZenithAngle;
       zenith->from = from;
       zenith->to   = to;
-      zenith->set(val);
+      zenith->set(val*GON_TO_RAD);
       zenith->from_dh = optional(g3->from_dh);
       zenith->to_dh   = optional(g3->to_dh);
       g3->obs_cluster->observation_list.push_back(zenith);  
@@ -814,6 +1054,9 @@ int DataParser::g3_obs_vector(const char *name)
       vector->to_dh   = optional(g3->to_dh);
 
       g3->obs_cluster->observation_list.push_back(vector);
+      g3->scale.push_back(1.0);
+      g3->scale.push_back(1.0);
+      g3->scale.push_back(1.0);
 
       return end_tag(name);
     }
@@ -827,19 +1070,220 @@ int DataParser::g3_obs_xyz(const char *name)
   stringstream istr(text_buffer);
   string id;
   double x, y, z;
-
+  
   if (pure_data(istr >> id >> x >> y >> z))
     {
       text_buffer.clear();
-
+      
       XYZ* xyz = new XYZ;
       xyz->id  = id;
       xyz->set_xyz(x, y, z);
 
       g3->obs_cluster->observation_list.push_back(xyz);
+      g3->scale.push_back(1.0);
+      g3->scale.push_back(1.0);
+      g3->scale.push_back(1.0);
 
       return end_tag(name);
     }
   
   return error("### bad <xyz>");
 }
+
+int DataParser::g3_obs_hdiff(const char *name)
+{
+  using namespace g3;  
+  stringstream istr(text_buffer);
+  string       from, to;
+  double       val;
+
+  if (pure_data(istr >> from >> to >> val))
+   {
+     text_buffer.clear();
+     
+     HeightDiff* hdiff = new HeightDiff;
+     hdiff->from = from;
+     hdiff->to   = to;
+     hdiff->set(val);
+     hdiff->from_dh = optional(g3->from_dh);
+     hdiff->to_dh   = optional(g3->to_dh);
+     g3->obs_cluster->observation_list.push_back(hdiff);  
+
+     return  end_tag(name);
+   }
+
+  return error("### bad <hdiff>");
+}
+
+int DataParser::g3_obs_height(const char *name)
+{
+  using namespace g3;  
+  stringstream istr(text_buffer);
+  string       id;
+  double       val;
+
+  if (pure_data(istr >> id >> val))
+   {
+     text_buffer.clear();
+     
+     Height* height = new Height;
+     height->id = id;
+     height->set(val);
+     g3->obs_cluster->observation_list.push_back(height);  
+
+     return  end_tag(name);
+   }
+
+  return error("### bad <height>");
+}
+
+int DataParser::g3_const_apriori_sd(const char *name)
+{
+  using namespace g3;  
+  stringstream istr(text_buffer);
+  double       sd;
+
+  if (pure_data(istr >> sd))
+   {
+     text_buffer.clear();
+ 
+     g3->model->set_apriori_sd(sd);
+     
+     return  end_tag(name);
+   }
+
+  return error("### bad <height>");
+}
+
+int DataParser::g3_const_conf_level(const char *name)
+{
+  using namespace g3;  
+  stringstream istr(text_buffer);
+  double       cl;
+
+  if (pure_data(istr >> cl))
+   {
+     text_buffer.clear();
+ 
+     g3->model->set_conf_level(cl);
+     
+     return  end_tag(name);
+   }
+
+  return error("### bad <confidence-level>");
+}
+
+int DataParser::g3_const_ellipsoid_id(const char *name)
+{
+  using namespace g3;  
+  stringstream istr(text_buffer);
+  string       s;
+
+  if (pure_data(istr >> s))
+   {
+     text_buffer.clear();
+     const char* const name = s.c_str();
+     gama_ellipsoid id = ellipsoid(name);
+
+     set(&g3->model->ellipsoid, id);
+
+     if (id != ellipsoid_unknown) return end_tag(name);
+   }
+
+  return error("### bad <ellipsoid> <id>");
+}
+
+int DataParser::g3_const_ellipsoid_b(const char *name)
+{
+  using namespace g3;  
+  stringstream istr(text_buffer);
+  double       a, b;
+
+  if (pure_data(istr >> a >> b))
+   {
+     text_buffer.clear();
+
+     set(&g3->model->ellipsoid, ellipsoid_unknown);
+     g3->model->ellipsoid.set_ab(a, b);
+
+     return end_tag(name);
+   }
+
+  return error("### bad <ellipsoid> <a> <b>");
+}
+
+int DataParser::g3_const_ellipsoid_inv_f(const char *name)
+{
+  using namespace g3;  
+  stringstream istr(text_buffer);
+  double       a, inv_f;
+
+  if (pure_data(istr >> a >> inv_f))
+   {
+     text_buffer.clear();
+
+     set(&g3->model->ellipsoid, ellipsoid_unknown);
+     g3->model->ellipsoid.set_af1(a, inv_f);
+
+     return end_tag(name);
+   }
+
+  return error("### bad <ellipsoid> <a> <inv-f>");
+}
+
+int DataParser::g3_const_ang_degrees(const char *name)
+{
+  text_buffer.clear();
+  g3->model->set_angular_units_degrees();
+  
+  return end_tag(name);
+}
+
+int DataParser::g3_const_ang_gons(const char *name)
+{
+  text_buffer.clear();
+  g3->model->set_angular_units_gons();
+  
+  return end_tag(name);
+}
+
+int DataParser::g3_obs_angle(const char *name)
+{
+  using namespace g3;  
+  stringstream istr(text_buffer);
+  string       from, left, right;
+  string       sval;
+
+  if (pure_data(istr >> from >> left >> right >> sval))
+   {
+     text_buffer.clear();
+     
+     double val;
+     if (deg2gon(sval, val))
+       {
+         g3->scale.push_back(3.0864);   // ss --> cc
+       }
+     else
+       {
+          istringstream istr(sval);
+          istr >> val;
+
+          g3->scale.push_back(1.0);
+       }
+
+     Angle* angle = new Angle;
+     angle->from  = from;
+     angle->left  = left;
+     angle->right = right;
+     angle->set(val*GON_TO_RAD);
+     angle->from_dh  = optional(g3->from_dh);
+     angle->left_dh  = optional(g3->to_dh);
+     angle->right_dh = optional(g3->to_dh);
+     g3->obs_cluster->observation_list.push_back(angle);  
+
+     return  end_tag(name);
+   }
+
+  return error("### bad <angle>");
+}
+
